from locals import *

from mapobject import MapObject

from util import render_image, render_text

from action import Action

class Base(MapObject):

    COST = {}
    COST['rock'] = (4, 0)
    COST['paper'] = (2, 1)
    COST['scissors'] = (6, 2)

    def __init__(self, x, y, owner, level):
        self.level = level
        MapObject.__init__(self, x, y, owner)
        self.object_name = 'base'
        self.buy_target1 = 'paper'
        self.buy_target2 = 'paper'
        self.buy_amount1 = 0
        self.buy_amount2 = 0

        self.buy_rect = [0, 0]
        self.dec_rect = [0, 0]
        self.inc_rect = [0, 0]
        self.buy_target_rect = [0, 0]
        return

    def render(self, screen, offset_x = 0, offset_y = 0):
        return MapObject.render(self, screen, offset_x, offset_y, \
        BUILDING_EMBLEM_POS)

    def render_infobox(self, screen, level):
        self.render_infobox_bg(screen, 0, 'Base owned by ' + self.owner)

        head = render_text('Conquer by occupying alone', 10)
        head_coords = (GUI_INFOBOX_COORDS[0] + 8, GUI_INFOBOX_COORDS[1] + 60)
        screen.blit(head, head_coords)

        head = render_text('at the end of the turn', 10)
        head_coords = (GUI_INFOBOX_COORDS[0] + 8, GUI_INFOBOX_COORDS[1] + 74)
        screen.blit(head, head_coords)

        head = render_text('Win by conquering all bases', 10)
        head_coords = (GUI_INFOBOX_COORDS[0] + 8, GUI_INFOBOX_COORDS[1] + 92)
        screen.blit(head, head_coords)

        #The code is a bit ugly, make generic code for both buying slots

        self.buybox1_rect = self.render_infobox_bg(screen, 1, 'Buy units 1')
        box_coords = self.buybox1_rect.topleft
        buy_target = self.get_buy_target((self.x, self.y + 1))
        if buy_target != None:
            self.buy_target1 = buy_target
        self.render_buy_controls(screen, self.buy_target1, self.buy_amount1, \
        box_coords, 0)

        self.buybox2_rect = self.render_infobox_bg(screen, 2, 'Buy units 2')
        box_coords = self.buybox2_rect.topleft
        buy_target = self.get_buy_target((self.x + 2, self.y + 1))
        if buy_target != None:
            self.buy_target2 = buy_target
        self.render_buy_controls(screen, self.buy_target2, self.buy_amount2, \
        box_coords, 1)

        return

    def get_buy_target(self, buy_coords):
        new_occup = self.level.find_by_tile_coords(buy_coords)
        buy_target = None

        if new_occup == None:
            if self.occupier1 != None:
                buy_target = 'cant_buy'
        else:
            if new_occup.owner == self.owner and new_occup.moves_used == 0:
                buy_target = self.occupier1.object_name
            else:
                buy_target = 'cant_buy'
        return buy_target

    def render_buy_controls(self, screen, buy_target, amount, box_coords, index):
        self.buy_target_rect[index] = render_image(screen, \
        'object_'+ buy_target + '.png',\
        (box_coords[0] + 8, box_coords[1] + 60))

        if buy_target != 'cant_buy':

            self.dec_rect[index] = render_image(screen, 'gui_button_dec.png',\
            (box_coords[0] + 8, box_coords[1] + 104))

            self.inc_rect[index] = render_image(screen, 'gui_button_inc.png',\
            (box_coords[0] + 80, box_coords[1] + 104))

            self.buy_rect[index] = render_image(screen, 'gui_button_buy.png',\
            (box_coords[0] + 120, box_coords[1] + 102))

            buy_amt_text = render_text(str(amount), 12)

            screen.blit(buy_amt_text, \
            (box_coords[0] + 34, box_coords[1] + 104))
        return

    def use_infobox_gui(self, event_pos, res):
        #The code is ugly, make generic code for both buying slots ASAP
        value = None
        action = None
        if self.buybox1_rect.collidepoint(event_pos) or \
        self.buybox2_rect.collidepoint(event_pos):
            value = True

        if self.inc_rect[0].collidepoint(event_pos):
            self.buy_amount1 += 1
            cost = self.buy_amount1 * Base.COST[self.buy_target1][0]
            if cost > res[Base.COST[self.buy_target1][1]]:
                self.buy_amount1 -= 1
        elif self.dec_rect[0].collidepoint(event_pos):
            if self.buy_amount1 > 0:
                self.buy_amount1 -= 1
        elif self.buy_target_rect[0].collidepoint(event_pos):
            if self.occupier1 == None and self.buy_target1 != 'cant_buy':
                self.buy_target1 = self.cycle_units(self.buy_target1)
                self.buy_amount1 = 0
        elif self.buy_rect[0].collidepoint(event_pos):
            if self.buy_amount1 > 0:
                action = Action(ACTION_BUY_PREFIX + self.buy_target1, \
                (self.x, self.y + 1), self.buy_amount1)
            self.buy_amount1 = 0
            self.buy_amount2 = 0
            return action

        if self.inc_rect[1].collidepoint(event_pos):
            self.buy_amount2 += 1
            cost = self.buy_amount2 * Base.COST[self.buy_target2][0]
            if cost > res[Base.COST[self.buy_target2][1]]:
                self.buy_amount2 -= 1
        elif self.dec_rect[1].collidepoint(event_pos):
            if self.buy_amount2 > 0:
                self.buy_amount2 -= 1
        elif self.buy_target_rect[1].collidepoint(event_pos):
            if self.occupier2 == None and self.buy_target2 != 'cant_buy':
                self.buy_target2 = self.cycle_units(self.buy_target2)
                self.buy_amount2 = 0
        elif self.buy_rect[1].collidepoint(event_pos):
            if self.buy_amount2 > 0:
                action = Action(ACTION_BUY_PREFIX + self.buy_target2, \
                (self.x + 2, self.y + 1), self.buy_amount2)
            self.buy_amount1 = 0
            self.buy_amount2 = 0
            return action

        return value

    def cycle_units(self, unit):
        if unit == 'rock':
            return 'paper'
        if unit == 'paper':
            return 'scissors'
        if unit == 'scissors':
            return 'rock'
        return 'rock'

    def new_turn(self):
        MapObject.new_turn(self)
        self.occupier1 = self.level.find_by_tile_coords((self.x, self.y + 1))
        if self.occupier1 != None:
            self.buy_target1 = self.occupier1.object_name
        else:
            self.buy_target1 = 'paper'
        self.occupier2 = self.level.find_by_tile_coords((self.x + 2, self.y + 1))
        if self.occupier2 != None:
            self.buy_target2 = self.occupier2.object_name
        else:
            self.buy_target2 = 'paper'
        return