"""A module for storing whole rounds on the server side
and seeing how they play out"""

from locals import *

from turn import Turn
from action import Action

from log import log_message

from level import Level

class Game_Round:

    def __init__(self):
        self.buys = []
        self.moves = []
        self.twists = 0
        self.faction_turns_received = []
        self.finished = False #Becomes True when everything has been received
                              #AND processed
        return

    def add_action_from_msg(self, msg, faction):
        """Adds an action parsed from a network message
        Returns True when all actions sent by the client are processed
        Parameter 'faction' is a faction object"""
        values = msg.split()
        action = Action(ACTION_BUY_PAPER)
        if len(values) > 3:
            action = Action(values[0], (int(values[1]), int(values[2])), \
            int(values[3]))
        elif len(values) > 2:
            action = Action(values[0], (int(values[1]), int(values[2])))
        elif values[0] == ACTION_END_TURN:
            self.faction_turn_received(faction)
            return True #Turn finished
        else:
            action = Action(values[0])
        self.add_action(action, faction)
        return False #Not finished yet

    def add_action(self, action, faction):
        """Adds an action.
        Parameter 'faction' is a faction object"""
        if action.get_type() == ACTION_TWIST:
            if faction.use_resources(COST_TWIST):
               self.twists += 1
            else:
               log_message('Faction: ' + faction.get_color() \
               + ' tried to use twist with not enough resources, cheating?')
        elif action.get_type() == ACTION_BUY_ROCK or \
        action.get_type() == ACTION_BUY_PAPER or \
        action.get_type() == ACTION_BUY_SCISSORS:
            self.buys.append((action, faction))
        elif action.get_type() == ACTION_MOVE_LEFT or \
        action.get_type() == ACTION_MOVE_RIGHT or \
        action.get_type() == ACTION_MOVE_UP or \
        action.get_type() == ACTION_MOVE_DOWN:
            self.moves.append((action, faction))
        return

    def faction_turn_received(self, faction):
        """Tell the round that the turn data from faction
        given as a parameter has been received.
        Checks if a client tries to send a turn a second time"""
        for f in self.faction_turns_received:
            if f == faction:
                 return False
        self.faction_turns_received.append(faction)
        return True

    def amount_factions_received(self):
        return len(self.faction_turns_received)

    def process_round(self, level):
        """Process one round of the turn-based game.
        turn_data is a list of actions players have made - there's
        one turn object each round for each player in the game.
        This function processes them according to simultaneous turn rules
        and returns the output to the clients one "move" at a time - this way,
        there might be player interactivity between turns as well,
        and clients get the data as soon as it is processed."""
        client_event = ACTION_BUY_PAPER
        if (self.twists % 2) == 1:
            log_message('Server twisting')
            self.twists = 0
            real_action = level.toggle_twist()
            return real_action.get_network_msg()
        if len(self.buys) > 0:
            b = self.buys.pop()
            real_action = level.buy(b[0], b[1].get_color())
            if real_action != None:
                return real_action.get_network_msg() + ' ' + b[1].get_color()
            else:
                return ''
        elif len(self.moves) > 0:
            m = self.moves[0]
            self.moves.remove(m)

            level.select_by_tile_coords(m[0].get_coords(), m[1].get_color())
            real_action = level.move(m[0].action_type)
            #Processing combat etc. goes here
            if real_action != None:
                return real_action.get_network_msg()
            else:
                return ''
        else:
            self.finished = True
            winner = level.new_turn()
            return ''