#This file contains constant values used throughout the game code.

#This is used for creating config file paths
#IMPORTANT: CHANGE THIS TO CORRESPOND TO YOUR GAME.
#NOT CHANGING THIS MIGHT LEAD TO CONFLICTING CONFIG FILES.
GAME_NAME_SHORT = "tubascrobast"

#Game screen dimensions used by functions in util.py
SCREEN_WIDTH = 800
SCREEN_HEIGHT = 600

#
#These are used by functions in util.py:
#

FPS = 30

#For the fade_to_black function.
FADE_STATE_NONE = 0       # The screen is not faded at all.
FADE_STATE_HALF = 128   # The screen is half faded.
FADE_STATE_BLACK = 255    # The screen is all black.

#Fade times in seconds, FADE_IN must be negative and FADE_OUT positive.
FADE_IN = -0.5
FADE_OUT = 0.5

#For the GUI
FONT_SIZE = 18
FONT_LINE_SPACING = 28
COLOR_GUI = (255, 255, 255)
COLOR_GUI_HILIGHT = (100, 100, 100)
COLOR_GUI_DARK = (100, 40, 40)
COLOR_GUI_BG = (0, 0, 0)

#For the menu, used in menu.py:
MENU_QUIT = -1
MENU_MAX_VISIBLE = 5 #maximum amount of choices visible in the menu at one time
GUI_MENU_TOP = 290 #in pixels
GUI_MENU_CENTER = 620 #in pixels

#How many old log lines are kept in the log file when the log is updated
MAX_OLD_LOG_LINES = 50
#How many bytes of log are kept in memory
MAX_LOG_BYTES = 65536
#How many bytes are cut off from the log when it becomes larger than
#MAX_LOG_BYTES
LOG_CUTOFF = 2048

GUI_INFOBOX_COORDS = (600, 34)
GUI_BUYBOX1_COORDS = (600, 184)
GUI_BUYBOX2_COORDS = (600, 334)

FACTION_RED = (200, 50, 50)
FACTION_PURPLE = (180, 50, 150)
FACTION_YELLOW = (180, 150, 50)
FACTION_BLACK = (120, 120, 120)
FACTION_WHITE = (240, 240, 240)
FACTION_UNOCCUPIED = (150, 150, 150)

ACTION_PREFIX = 'act_'
ACTION_BUY_PREFIX = ACTION_PREFIX + 'buy_'
ACTION_BUY_ROCK = ACTION_PREFIX + 'buy_rock'
ACTION_BUY_PAPER = ACTION_PREFIX + 'buy_paper'
ACTION_BUY_SCISSORS = ACTION_PREFIX + 'buy_scissors'
ACTION_MOVE_PREFIX = ACTION_PREFIX + 'move_'
ACTION_MOVE_LEFT = ACTION_PREFIX + 'move_left'
ACTION_MOVE_RIGHT = ACTION_PREFIX + 'move_right'
ACTION_MOVE_UP = ACTION_PREFIX + 'move_up'
ACTION_MOVE_DOWN = ACTION_PREFIX + 'move_down'
ACTION_SPLIT = ACTION_PREFIX + 'split'
ACTION_JOIN = ACTION_PREFIX + 'join'
ACTION_TWIST = ACTION_PREFIX + 'twist'
ACTION_START_TURN = ACTION_PREFIX + 'start_turn'
ACTION_END_TURN = ACTION_PREFIX + 'end_turn'

MONEY_ZERO = (0,0,0)
COST_TWIST = (20,20,20)
SHRINE_PRODUCTION = 10

LOCAL = 0
PUBLIC = 1
PLAYER_LIMIT = 5

MATCHMAKING_SERVER_ADDRESS = 'http://tubascrobast.hectigo.net/'
PORT_DEFAULT = 37655
PORT_RANGE = 5 #The number of consecutive ports the program tries to use
PACKET_MAX_SIZE = 1024
SERVER_BACKLOG = 5

PLAYER = 0
AI = 1

FILE_EXT_LEVEL = '.txt'

CHAT_MSG_MAX_LEN = 100
NICK_MAX_LEN = 32

TILE_WALL = 1
TILE_BLOCKED = 2

#Network grammar:
COMMAND_PING = '00-ping'
COMMAND_GET_ID = '01-getid'
COMMAND_DISCONNECT = '02-disconnect'
COMMAND_TERMINATE = '03-terminate'
COMMAND_KILLYOURSELF = '04-kill_yourself'
COMMAND_QUERY = '10-query'
COMMAND_CHAT = '11-chat'
COMMAND_SET_NICK = '12-nick'

GENERIC_RESPONSE = '-response'

RESPONSE_PING = COMMAND_PING + GENERIC_RESPONSE
RESPONSE_TERMINATE = COMMAND_TERMINATE + GENERIC_RESPONSE
RESPONSE_GET_ID = COMMAND_GET_ID + GENERIC_RESPONSE
RESPONSE_DISCONNECT = COMMAND_DISCONNECT + GENERIC_RESPONSE

RESPONSE_UNKNOWN_COMMAND = 'unknown server command: '
RESPONSE_AUTH_FAILURE = 'authorization failure: '

GAME_COMMAND_ASK_LEVEL = '20-ask_level'
GAME_COMMAND_ASK_FACTION = '21-ask_faction'
GAME_COMMAND_SEND_TURN = '22-turn-data'
GAME_COMMAND_SET_RES = '23-set-resources'
#Network grammar end

SERVER_RATE = 500.0 #Maximum messages processed by the server in a second

NO_AVAILABLE_FACTIONS = 'no_factions'

TILE_WIDTH = 32
TILE_HEIGHT = 32

BUILDING_EMBLEM_POS = (39, 5)
