"""Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
"""

import os
import random

import pygame
from pygame.locals import *

from util import Util, parse_config, write_config, write_log, render_image, ask_input
from log import error_message
import data

from menu import Menu
from client import Client
from server import Server
from locals import *

def main():

    # These should always be run first for the included utility modules to work
    parse_config()
    pygame.init()

    screen = pygame.display.set_mode((SCREEN_WIDTH,SCREEN_HEIGHT))

    pygame.display.set_caption('Tubasc Robast')

    render_image(screen, 'game-background.jpg')
    render_image(screen, 'game-decoration.png')

    m = Menu(screen, ['Single player', 'Host a multiplayer game',
    'Join a multiplayer game', 'Quit'], screen, 'Main Menu')

    quit = False

    while not quit:
        menu_choice = m.run()
        if (menu_choice == 0):
          #Single player game
          server_key = str(random.random())
          #Map selection goes here
          current_server = Server(LOCAL, PORT_DEFAULT, server_key, '2p_clash')
          current_server.start()
          current_client = Client(screen, 'localhost', PORT_DEFAULT, server_key)
          current_client.run()

          current_server.join()
        elif (menu_choice == 1):
          #Hosting a multiplayer game
          server_key = str(random.random())
          #Level selection
          level_found = True
          level_name = ask_input(screen, 'Select level:', '2p_clash')
          #A more sophisticated level menu to be done later
          try:
              current_server = Server(PUBLIC, PORT_DEFAULT, server_key, level_name)
          except:
              level_found = False
              error_message('Selected level not found!')
          if level_found:
              current_server.start()
              #Waiting for client connections goes here
              current_client = Client(screen, 'localhost', PORT_DEFAULT, server_key)
              current_client.run()

              current_server.join()
        elif (menu_choice == 2):
          #Joining a multiplayer game
          address = ask_input(screen, 'Input host IP address:', '127.0.0.1')
          #Matchmaking to be done later
          if (input_addr != None):
              current_client = Client(screen, address, PORT_DEFAULT)
              current_client.run()
        elif (menu_choice == 3 or menu_choice == MENU_QUIT):
          quit = True

    # Save the config and log on exit!
    write_config()
    write_log()
    return

