import pygame

from locals import *

from util import render_image, render_text

from log import log_message

class MapObject:

    def __init__(self, x, y, owner):
        self.x = x
        self.y = y
        self.render_width = 0
        self.render_height = 0
        self.owner = owner
        self.object_name = 'default'
        self.selected = False
        self.rect = pygame.Rect(0, 0, 0, 0)
        self.max_moves = 0
        self.new_turn()
        self.dead = False
        return

    def render(self, screen, offset_x = 0, offset_y = 0, emblem_pos = None):
        #Check if the object is on screen:
        if (self.x * TILE_WIDTH + offset_x + self.render_width) < 0 or \
        (self.y * TILE_HEIGHT + offset_y + self.render_height) < 0 or \
        (self.x * TILE_WIDTH + offset_x) > SCREEN_WIDTH or \
        (self.y * TILE_HEIGHT + offset_y) > SCREEN_HEIGHT:
            return False
        #Rendering here
        self.rect = render_image(screen, 'object_' + self.object_name + '.png',\
        (self.x * TILE_WIDTH + offset_x, self.y * TILE_HEIGHT + offset_y))

        self.render_width = self.rect.width
        self.render_height = self.rect.height

        if emblem_pos == None:
            render_image(screen, 'emblem_small_' + self.owner + '.png',\
            (self.x * TILE_WIDTH + offset_x, self.y * TILE_HEIGHT + offset_y))
        else:
            render_image(screen, 'emblem_large_' + self.owner + '.png',\
            (self.x * TILE_WIDTH + offset_x + emblem_pos[0], \
            self.y * TILE_HEIGHT + offset_y + emblem_pos[1]))

        if self.selected:
            pygame.draw.rect(screen, COLOR_GUI, self.rect, 1)
        return True
        
    def render_infobox(self, screen, level):
        return
        
    def render_infobox_bg(self, screen, index, title = ''):
        rect = render_image(screen, 'infobox.png', (GUI_INFOBOX_COORDS[0], \
        GUI_INFOBOX_COORDS[1] + index*150))
        head = render_text(title, 11)
        head_coords = (GUI_INFOBOX_COORDS[0] + 8, GUI_INFOBOX_COORDS[1] + 38 \
        + index*150)
        screen.blit(head, head_coords)
        return rect

    def produce_resources(self):
        return MONEY_ZERO

    def move(self, action_type):
        if self.moves_used < self.max_moves:
            self.moves_used += 1
        else:
            return False
        if action_type == ACTION_MOVE_LEFT:
            self.x = self.x - 1
        elif action_type == ACTION_MOVE_RIGHT:
            self.x = self.x + 1
        elif action_type == ACTION_MOVE_UP:
            self.y = self.y - 1
        elif action_type == ACTION_MOVE_DOWN:
            self.y = self.y + 1
        return True

    def reset_last_turn(self):
        self.x = self.startpos[0]
        self.y = self.startpos[1]
        self.moves_used = 0
        self.twisted = False
        return

    def new_turn(self):
        self.moves_used = 0
        self.startpos = (self.x, self.y)
        self.twisted = False
        return

    def toggle_twist(self):
        self.twisted = not self.twisted
        log_message('object is twisted: ' + str(self.twisted))
        return