"""Network utility function module"""

from locals import *

import socket
import time
from log import log_message, error_message

#Functions for testing network operation and connection management:

def ping(sock):
    ping = 0
    start_time = time.clock()
    try:
        sock.send(COMMAND_PING)
        data = sock.recv(PACKET_MAX_SIZE)
        if (data.startswith(RESPONSE_PING)):
            ping = int((time.clock() - start_time)*1000)
            log_message('Client ping: ' + str(ping) + "ms")
            return ping

    except socket.error, (value,message):
        error_message(message)
    return -1

def get_client_id(sock, client_nick = 'Anonymous'):
    try:
        sock.send(COMMAND_GET_ID + ' ' + client_nick)
        data = sock.recv(PACKET_MAX_SIZE)
        if (data.startswith(RESPONSE_GET_ID)):
            values = data.split()
            response = values[0]
            client_id = int(values[1])
            log_message('Client received ID: ' + str(client_id))
            return client_id

    except socket.error, (value,message):
        error_message(message)
    return -1

def disconnect(sock, client_id):
    try:
        sock.send(COMMAND_DISCONNECT + ' ' + str(client_id))
        data = sock.recv(PACKET_MAX_SIZE)
        if (data.startswith(RESPONSE_DISCONNECT)):
            log_message('Client ' + str(client_id) + ' disconnected')
            return True

    except socket.error, (value,message):
        error_message(message)
    return False

def terminate_server(sock, server_key = None):
    if server_key == None:
        error_message('Client: Not enough user privileges to terminate server')
        return False

    log_message('Client: Sending terminate command to server')
    try:
        terminate_command = COMMAND_TERMINATE + ' ' + server_key
        sock.send(terminate_command)
        data = sock.recv(PACKET_MAX_SIZE)
        log_message('Client received: ' + data)
        if (data.startswith(RESPONSE_TERMINATE)):
            return True

    except socket.error, (value,message):
        error_message(message)
    return False

def network_command(sock, client_id, command, params):
    """Generic network command after the client-server
    connection with ID has been established"""
    try:
        sock.send(str(client_id) + ' ' + command + ' ' + params)
        data = sock.recv(PACKET_MAX_SIZE)
        if (data.startswith(command+GENERIC_RESPONSE)):
            response, empty, message = data.partition(' ')
            #log_message('Client sent command: ' + command + ' got ' + message)
            return message

    except socket.error, (value,message):
        error_message(message)
    return False

#Functions using the generic network command format:

def query_events(sock, client_id):
    message = network_command(sock, client_id, COMMAND_QUERY, '')
    if message:
        event_queue, empty, message = message.partition(' ')
        return (int(event_queue), message)
    else:
        return (-1, None)

def submit_chat(sock, client_id, message):
    return network_command(sock, client_id, COMMAND_CHAT, message)

def set_nick(sock, client_id, nick):
    return network_command(sock, client_id, COMMAND_SET_NICK, nick)
