"""A module for storing a turns of players"""

from locals import *

from action import Action

class Turn:

    def __init__(self):
        self.actions = []
        self.twist = False
        self.active = True
        return

    def add_action(self, action):
        if self.active:
            if action.action_type == ACTION_TWIST:
                self.toggle_twist()
            else:
                self.actions.append(action)
            return True
        else:
            return False

    def undo_action(self):
        if self.active:
            return self.actions.pop()
        else:
            return None

    def toggle_twist(self):
        return self.set_twist(not self.twist)

    def set_twist(self, twist):
        if self.active:
            self.twist = twist
            return True
        else:
            return False

    def get_twist(self):
        return self.twist

    def end(self):
        self.active = False #Turn finished - player can't do any more changes
        return

    def get_network_msgs(self):
        if self.active:
            return None
        msgs = []
        msgs.append(ACTION_START_TURN)
        if self.twist:
            msgs.append(ACTION_TWIST)
        for a in self.actions:
            msgs.append(a.get_network_msg())
        msgs.append(ACTION_END_TURN + ' ' + str(len(msgs) + 1))
        return msgs