"""Stores information about a moving unit"""

from locals import *

from mapobject import MapObject

from util import render_image, render_text

from log import log_message

class Unit(MapObject):

    MAX_HPS = {}
    MAX_HPS['paper'] = 8
    MAX_HPS['scissors'] = 16
    MAX_HPS['rock'] = 24

    MAX_MOVES = {}
    MAX_MOVES['paper'] = 5
    MAX_MOVES['scissors'] = 4
    MAX_MOVES['rock'] = 3

    DAMAGE = {}
    #First: attacker, second: target
    DAMAGE[('rock', 'rock')] = 6
    DAMAGE[('rock', 'paper')] = 2
    DAMAGE[('rock', 'scissors')] = 8
    DAMAGE[('paper', 'rock')] = 6
    DAMAGE[('paper', 'paper')] = 2
    DAMAGE[('paper', 'scissors')] = 1
    DAMAGE[('scissors', 'rock')] = 4
    DAMAGE[('scissors', 'paper')] = 12
    DAMAGE[('scissors', 'scissors')] = 6

    TWISTED_DAMAGE = {}
    TWISTED_DAMAGE[('rock', 'rock')] = 6
    TWISTED_DAMAGE[('rock', 'paper')] = 8
    TWISTED_DAMAGE[('rock', 'scissors')] = 2
    TWISTED_DAMAGE[('paper', 'rock')] = 2
    TWISTED_DAMAGE[('paper', 'paper')] = 2
    TWISTED_DAMAGE[('paper', 'scissors')] = 4
    TWISTED_DAMAGE[('scissors', 'rock')] = 16
    TWISTED_DAMAGE[('scissors', 'paper')] = 3
    TWISTED_DAMAGE[('scissors', 'scissors')] = 6

    def __init__(self, x, y, owner = 'red', unit_class = 'paper', amount = 1):
        self.amount = int(amount)
        self.unit_class = unit_class
        self.max_hp = Unit.MAX_HPS[unit_class]
        self.top_hp = self.max_hp
        MapObject.__init__(self, x, y, owner)
        self.object_name = unit_class
        self.max_moves = Unit.MAX_MOVES[self.unit_class]
        return

    def get_description(self):
        description = self.size_descriptor() + self.group_descriptor() + \
        ' of ' + self.unit_class
        return description

    def render(self, screen, offset_x = 0, offset_y = 0):
        if MapObject.render(self, screen, offset_x, offset_y):
            amt_text = render_text(str(self.amount), 9)
            screen.blit(amt_text, (self.x * TILE_WIDTH + offset_x, \
            self.y * TILE_HEIGHT + offset_y + self.render_height - 12))
        return

    def render_infobox(self, screen, level):
        self.render_infobox_bg(screen, 0, self.get_description())

        head = render_text('Move with the arrow keys', 10)
        head_coords = (GUI_INFOBOX_COORDS[0] + 8, GUI_INFOBOX_COORDS[1] + 60)
        screen.blit(head, head_coords)
        
        head = render_text('Moves used: ' + str(self.moves_used) + '/' + str(self.max_moves), 10)
        head_coords = (GUI_INFOBOX_COORDS[0] + 8, GUI_INFOBOX_COORDS[1] + 78)
        screen.blit(head, head_coords)
        return

    def group_descriptor(self):
        if (self.unit_class == 'rock'):
            return ' pile'
        elif (self.unit_class == 'paper'):
            return ' stack'
        elif (self.unit_class == 'scissors'):
            return ' group'
        return ''

    def size_descriptor(self):
        if (self.amount < 6):
            return 'A tiny'
        elif (self.amount < 11):
            return 'A small'
        elif (self.amount < 21):
            return 'A large'
        elif (self.amount < 51):
            return 'A great'
        elif (self.amount < 101):
            return 'An enormous'
        elif (self.amount < 501):
            return 'A humongous'
        else:
            return 'A gigantic'
        return ''

    def attack(self, target):
        """Make the target object take damage."""
        log_message(self.object_name + ' attacking ' + target.object_name)
        self.moves_used = self.max_moves
        if self.twisted:
            log_message('twisted damage')
            damage = Unit.TWISTED_DAMAGE[(self.unit_class, target.unit_class)]
        else:
            log_message('normal damage')
            damage = Unit.DAMAGE[(self.unit_class, target.unit_class)]
        log_message('damage: ' + str(damage * self.startamount))
        target.take_damage(damage * self.startamount)
        return

    def take_damage(self, damage):
        if self.top_hp < damage:
            #Kill the topmost unit:
            damage = damage - self.top_hp
            self.top_hp = self.max_hp
            self.amount -= 1
            #Kill other units:
            killed_amount = damage / self.max_hp
            self.amount -= killed_amount
            damage = damage - self.max_hp * killed_amount
            if self.amount < 1:
                self.amount = 0
        #Damage the topmost unit:
        self.top_hp -= damage
        return

    def reset_last_turn(self):
        MapObject.reset_last_turn(self)
        self.amount = self.startamount
        self.top_hp = self.start_top_hp
        self.dead = False
        return

    def new_turn(self):
        MapObject.new_turn(self)
        self.startamount = self.amount
        self.start_top_hp = self.top_hp
        if self.amount < 1:
            self.amount = 0
            self.dead = True
        return